<?php

namespace FiloBlu\Flow\Model\Actions\From;

use Magento\Catalog\Model\Product;
use Magento\Store\Model\StoreRepository;
use Magento\Catalog\Model\ProductFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable as ProductTypeConfigurable;
use Magento\Store\Model\WebsiteRepository;
use Magento\Catalog\Api\ProductRepositoryInterface;

/**
 * Class Reassignwebsite
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Reassignwebsite
{

    /**
     * @var string
     */
    CONST ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * @var Product
     */
    protected $productFactory;

    /**
     * @var array|StoreRepository
     */
    protected $websites = [];

    /**
     * @var WebsiteRepository
     */
    protected $websiteRepository;

    /**
     * @var ProductTypeConfigurable
     */
    protected $productTypeConfigurable;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var bool
     */
    public $useParent = false;


    /**
     * @param ProductFactory $productFactory
     * @param WebsiteRepository $websiteRepository
     * @param ProductTypeConfigurable $productTypeConfigurable
     * @param ProductRepositoryInterface $productRepository
     */
    public function __construct(
        ProductFactory $productFactory,
        WebsiteRepository $websiteRepository,
        ProductTypeConfigurable $productTypeConfigurable,
        ProductRepositoryInterface $productRepository
    )
    {
        $this->productFactory = $productFactory;
        $this->websiteRepository = $websiteRepository;
        $this->productTypeConfigurable = $productTypeConfigurable;
        $this->productRepository = $productRepository;
        $this->websites = $this->getWebsites();
    }


    /**
     * @return array
     */
    public function getWebsites()
    {
        $websites = $this->websiteRepository->getList();
        $websiteIdArray = [];
        $websiteCodeArray = [];
        foreach ($websites as $website) {
            $websiteIdArray[$website['website_id']] = $website['website_id'];
            $websiteCodeArray[$website['code']] = $website['website_id'];
        }

        return ['id' => $websiteIdArray, 'code' => $websiteCodeArray];
    }


    /**
     * @param $prod
     * @param $value
     * @param $storeId
     * @return false|void
     */
    public function process($prod, $value, $storeId)
    {
        if(!$value) {
            return false;
        }

        $websites = explode(',', str_replace('|', ',', $value));

        if (count($websites) === 0) {
            return false;
        }

        $websiteToProcess = $this->getNormalizedWebsiteArray($websites);

        if (count($websiteToProcess)) {
            $product_id = $this->productFactory->create()->getIdBySku($prod->getSku());
            $product = $this->productFactory->create()->setStoreId(0)->load($product_id);

            // Switch to the parent?
            if ($this->useParent){
                $parentIds = $this->productTypeConfigurable->getParentIdsByChild($product_id);
                if (isset($parentIds[0]) && $parentIds[0] > 0){
                    $product = $this->productFactory->create()->setStoreId(0)->load($parentIds[0]);
                }
            }

            $productFromRepository = $this->productRepository->get($product->getSku());
            $productFromRepository->setWebsiteIds(array_diff($product->getWebsiteIds(), $product->getWebsiteIds()));
            $productFromRepository->setWebsiteIds($websiteToProcess);
            $productFromRepository->save();
        }
    }

    /**
     * @param $websites
     * @return array
     */
    protected function getNormalizedWebsiteArray($websites)
    {
        $websiteToProcess = [];
        foreach ($websites as $website) {
            $website = trim($website ??'');
            if (is_numeric($website)) {
                if (key_exists($website, $this->websites['id'])) {
                    $websiteToProcess[] = $this->websites['id'][$website];
                }
            } else {
                if (key_exists($website, $this->websites['code'])) {
                    $websiteToProcess[] = $this->websites['code'][$website];
                }
            }
        }

        return array_unique($websiteToProcess);
    }

    public function setUseParent($bool = true){
        $this->useParent = $bool;
    }

}
